#pragma once
#ifndef S_SIF1_ENC_LIB
#define S_SIF1_ENC_LIB

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#include "../SIF_1_lib/sif_version_n.h"

	typedef void* SIF_ENC_HANDLE;


	typedef enum CompMode {
		qv_b = 0,
		vbr_1_p = 1,
		vbr_2_p = 2,
		abr = 3,
		cbr = 4,
		vbr_1_p_ext = 5, // Additional first pass with collecting bitrate data
	} CompMode;

	typedef enum ME_Mode {
		me_ultrafast = 0,
		me_faster = 2,
		me_fast = 3,
		me_standard = 4,
		me_slow = 5,
		me_slower = 6,
	} ME_Mode;

	typedef enum SubME_Mode {
		sub_fastest = 0,
		sub_fast = 1,
		sub_medium = 2,
		sub_slow = 3,
	} SubME_Mode;

	typedef enum SIF_Presets {
		fast = 4,
		def = 5,
	} SIF_Presets;

	typedef enum SIF_Mode {
		sif_fast = 0,
		sif_medium = 1,
		sif_slow = 2,
	} SIF_Mode;

	typedef enum InputFormat {
		IF_RGB_24 = 1,
		IF_RGB_32 = 2,
		IF_YUY2 = 3,
		IF_YV12 = 4,
		IF_I420 = 5,
		IF_I420p10 = 6,
		IF_P010 = 7,
		IF_I420p16 = 8,
	} InputFormat;

	typedef enum SubPelMode {
		SPM_half_p = 0,
		SPM_Q_pel = 1,
		SPM_Q_Reserved1 = 2,
		SPM_Q_Reserved2 = 3,
	} SubPelMode;

	typedef struct UstanovkiME {
		ME_Mode me_m;
		SubME_Mode sub_me_m;
		SubPelMode pix_m; // 2
		int32_t chroma_me;
	} UstanovkiME;

	typedef enum FirstPassMode {
		pm_slow = 0,
		pm_medium = 1,
		pm_fast = 2,
	} FirstPassMode;


	typedef struct UstanovkiSjatia {
		int32_t bitrate; // Current bitrate of compression
		int32_t porog; // Current threshold of compression
		int32_t vb_porog; // Threshold of the first pass of multi pass compression
		int32_t ab_porog; // Compression threshold for ABR mode
		int32_t podjim_col; // Multiplier ratio for color components comparing to brightness
		int32_t podjim_kvan; // Multiplier ratio for texture quantization threshold 
		int32_t chisl_delt_kadr; // Max interval between the key frames
		int32_t min_r_key_fr; // Minimal interval between the key frames
		int32_t buf_size; // The size of the buffer to control bitrate
		CompMode ter_rej; // Current compression mode
		int8_t stat_f[MAX_PATH];
		UstanovkiME me_method; // Motion detector settings
		SIF_Mode sif_m; // Mode of an analyzing part of SIF transform
		EntropyMode entr_coder; // Number of threads in entropy encoder
		int32_t auto_en_mode; // Auto-mode to define the number of threads for entropy encoder
		FirstPassMode pass_m; // Fast first pass indicator
		int32_t ref_fr_n; // Number of reference frames
		CollorSpaceMode colSp; // 3
		CollorResolution colRes; //2 // only 420 support
		PixelPrecision bit_Dep; //2 
		int32_t interlive_mode; // Source with interleaving // 0 - Off
		int32_t field_ord; // Field order 0 -Top field first, 1-Bottom field first.
		int32_t no_rf_mod; // Turn off reference frames modulation 0 - On, 1- Off
		VideoTransferFunction colVTS; // Additional parameters for transmission curve
		VideoPrimaries colVP; // Source additional parameters
		NominalRange colNR; // Additional field for render range
	} UstanovkiSjatia;

	typedef struct SifExtSetting {
		UstanovkiSjatia cs;
		InputFormat ifrm;
		int32_t x, y; // Image size
		int32_t fr_rate; // Frame frequency nominator
		int32_t fr_scale; // Frame frequency denominator
		int32_t enc_fr_rate; // Record frame frequency to header 1 - On, 0 - Off
		int32_t tr_num; // Number of supported parallel threads
		int32_t* vbr_fr_buf; // Pointer to external buffer of VBR data of the first pass // Buffer has sizes of first pass frames 
		int32_t calc_metrics; // Calc objective metrics: 1 - On, 0 - Off
		int32_t show_stat; // Show encoding statistics: 1 - On, 0 - Off // Turned off for now
		OptimizationMode cpu;
		int32_t AspectRatioX;  // Aspect ratio nominator // max 8 bit // 0 - Off
		int32_t AspectRatioY;  // Aspect ration denominator
	} SifExtSetting;


	DLL_EXPORT_IMPORT(size_t, se_get_h_size)();

	DLL_EXPORT_IMPORT(void, se_get_st_setting)(struct SifExtSetting* init_s, SIF_Presets presets);

	DLL_EXPORT_IMPORT(SIF_ENC_HANDLE, se_init_s_handle)(uint8_t* h_data, SifExtSetting* init_s);

	DLL_EXPORT_IMPORT(size_t, se_get_cur_mem_size)(SIF_ENC_HANDLE sh, SifExtSetting* init_s);

	DLL_EXPORT_IMPORT(void, se_mod_compr_set)(SIF_ENC_HANDLE sh, UstanovkiSjatia* new_s);

	DLL_EXPORT_IMPORT(int32_t, se_init_mem_data)(SIF_ENC_HANDLE sh, uint8_t* s_data);

	DLL_EXPORT_IMPORT(int32_t, se_process_enc_frame)(SIF_ENC_HANDLE sh, uint8_t* in_data, int32_t* is_key, uint8_t* out_data);

	DLL_EXPORT_IMPORT(void, se_set_input_format)(SIF_ENC_HANDLE sh, InputFormat in_fr);

	DLL_EXPORT_IMPORT(void, se_set_input_pitch)(SIF_ENC_HANDLE sh, int32_t* ptch, int32_t plane_n);

	DLL_EXPORT_IMPORT(int32_t, se_stop_encoder)(SIF_ENC_HANDLE sh);

	DLL_EXPORT_IMPORT(int32_t, se_test_secondary_pass_encoder)(SIF_ENC_HANDLE sh, SifExtSetting* init_s);

	DLL_EXPORT_IMPORT(int32_t, se_get_vbr_por)(SIF_ENC_HANDLE sh);

	DLL_EXPORT_IMPORT(int32_t, se_get_version_n)();


#ifdef __cplusplus
}
#endif

#endif //S_SIF1_ENC_LIB
